const jsx = require('babel-plugin-syntax-jsx');

module.exports = function({ Plugin, types: t }) {
    const visitor = {};
    visitor.ArrowFunctionExpression = function(path) {
        const filename = this.file.opts.filename;

        if (path.node.type !== 'ArrowFunctionExpression') return;
        const arrowFunction = path.node;

        const id = path.parentPath.node.id;
        const className = id ? id.name : '__Clazz__';
        const isDeclaredAsConstant = id !== undefined;

        const properties = arrowFunction.params[0].properties.map(function(prop) {
            return prop.key;
        });

        const returnStatement = t.returnStatement(t.parenthesizedExpression(arrowFunction.body));

        const declares = properties.map(function(p) {
            return t.variableDeclarator(
                p,
                t.memberExpression(t.memberExpression(t.identifier('this'), t.identifier('props')), p)
            );
        });
        const destructProps = t.variableDeclaration('const', declares);

        const renderBody = t.blockStatement([destructProps, returnStatement]);
        const renderMethod = t.classMethod('method', t.identifier('render'), [], renderBody);
        const classBody = t.classBody([renderMethod]);
        const classDeclaration = t.classDeclaration(
            t.identifier(className),
            t.identifier('require("react").Component'),
            classBody,
            []
        );

        if (isDeclaredAsConstant) {
            path.parentPath.parentPath.replaceWith(classDeclaration);
        } else {
            const exporDefault = t.exportDefaultDeclaration(classDeclaration);
            path.parentPath.replaceWith(exporDefault);
        }
    };

    return {
        inherits: jsx,
        visitor
    };
};
